package FromDualMySQLagent;

use strict;
use warnings;
use Time::HiRes qw( gettimeofday );

use constant DBG  => 4;
use constant INFO => 1;
use constant WARN => 2;
use constant ERR  => 3;

sub readConfigurationFile
{
  my $file = $_[0];

  if ( ! -e "$file" ) {
    print "Configuration file " . $file . " does not exist.\n";
    exit 1;
  }

  my @config;
  open CONFIG, $file or die $!;
  while ( <CONFIG> ) {
    chomp;                  # remove newlines
    s/^\s*#.*//;            # remove comment
    s/^\s+//;               # remove leading white space
    s/\s+$//;               # remove trailing white space
    next unless length;     # anything left?
    push(@config, $_);
  }
  close(CONFIG);
  return @config;
}

sub getSections
{
  my @config = @_;

  my @sections;
  foreach my $line ( @config ) {

#     if ( $line =~ /^\[([\w-]+)\]/ ) {
    if ( $line =~ /^\[(.+)\]/ ) {
      push(@sections, $1);
    }
  }

  return @sections;
}

sub readSection
{
  my $config = shift;
  my $section = shift;

  my %section;
  my $in_section = 0;
  foreach my $line ( @$config ) {

#     if ( $line =~ /^\[(\w+)\]/ ) {
    if ( $line =~ /^\[(.+)\]/ ) {
      if ( $1 eq $section ) {
        $in_section = 1;
        next;
      }
      else {
        if ( $in_section == 1 ) {
          last;
        }
        $in_section = 0;
        next;
      }
    }

    if ( $in_section && ( $line =~ /^(\S+)\s*=\s*(.+)\s*$/ ) ) {
      $section{$1} = $2;
    }
  }

  return %section;
}

sub getDefaultParameter
{
  my $config = shift;

  my %hDefaults;

  # Hard coded defaults

  $hDefaults{'ClusterLog'}    = '/var/lib/mysql-cluster/ndb_1_cluster.log';
  $hDefaults{'Debug'}         = 0;
  $hDefaults{'Disabled'}      = 'false';
  $hDefaults{'FetchMethod'}   = 'DBI';
  $hDefaults{'LogFile'}       = '/var/log/zabbix/FromDualMySQLagent.log';
  $hDefaults{'Modules'}       = 'mysql myisam process';
  $hDefaults{'MysqlHost'}     = '127.0.0.1';
  $hDefaults{'MysqlPort'}     = '3306';
  $hDefaults{'Password'}      = '';
  $hDefaults{'PidFile'}       = '';
  $hDefaults{'Socket'}        = '/tmp/mysql.sock';
  $hDefaults{'Type'}          = 'mysqld';
  $hDefaults{'Username'}      = 'root';
  $hDefaults{'ZabbixServer'}  = '';

  # Read default from configuration
  my %default = &readSection(\@$config, 'default');

  foreach my $key ( keys %default ) {

    if ( $default{$key} ne '' ) {
      $hDefaults{$key} = $default{$key};
    }
  }

  return %hDefaults
}

sub getParameter
{
  my $config = shift;
  my $section = shift;

  my %defaults = getDefaultParameter(\@$config);

  if ( $defaults{'Debug'} == WARN ) {
    &log($defaults{'LogFile'}, INFO, "The defaults of this configuration file are:");
    foreach my $p ( keys %defaults ) {
      &FromDualMySQLagent::log($defaults{'LogFile'}, DBG, "$p - $defaults{$p}");
    }
  }

  my %parameter = readSection(\@$config, $section);

  if ( $defaults{'Debug'} == WARN ) {
    &log($defaults{'LogFile'}, INFO, "The values of section $section are:");
    foreach my $p ( keys %parameter ) {
      &FromDualMySQLagent::log($defaults{'LogFile'}, INFO, "$p - $parameter{$p}");
    }
  }

  foreach my $key ( keys %parameter ) {

    if ( $parameter{$key} ne '' ) {
      $defaults{$key} = $parameter{$key};
    }
  }
  $defaults{'Hostname'} = $section;

  if ( $defaults{'Debug'} == WARN ) {
    &log($defaults{'LogFile'}, INFO, "Which results in parameters for section $section:");
    foreach my $p ( keys %defaults ) {
      &FromDualMySQLagent::log($defaults{'LogFile'}, DBG, "$p - $defaults{$p}");
    }
  }
  return %defaults;
}

sub sendData {

  my $conf_ref = shift;
  my $values_ref = shift;
  my $list_ref = shift;
  my $domain = shift;

  if ( $$conf_ref{'Debug'} == DBG ) {
    &log($$conf_ref{'LogFile'}, DBG, 'Configuration:');
  }

  my ($cmd, $ret) = (undef, 0);
  foreach my $key ( keys %$values_ref ) {

    # in_array ?
    if(grep $_ eq $key, @$list_ref) {
      $cmd = "zabbix_sender -z " . $$conf_ref{'ZabbixServer'}. " -p 10051 -s '" . $$conf_ref{'Hostname'}. "' -k $domain.$key -o '" . $$values_ref{$key} . "'";

      if ( $$conf_ref{'Debug'} == DBG ) {
        &log($$conf_ref{'LogFile'}, DBG, "$cmd");
      }
      system("$cmd 2>&1 >>/dev/null");
      if ( $? != 0 ) {
        &log($$conf_ref{'LogFile'}, ERR, "$cmd failed.");
        $ret++;
      }
    }
  }
  return $ret;
}

sub log
{
  my $logfile = shift;
  my $lvl = shift;
  my $str = shift;

  if ( (! defined $logfile) || ($logfile eq '') ) {
    print "Logfile is not defined or empty.\n";
    exit 3;
  }

  if ( ! -w $logfile ) {
    print "Cannot write to logfile $logfile. Please check permissions.\n";
    exit 4;
  }

  my $severity = '';
  if ( $lvl == INFO ) {
    $severity = 'INFO';
  }
  elsif ( $lvl == WARN ) {
    $severity = 'WARN';
  }
  elsif ( $lvl == ERR ) {
    $severity = 'ERR ';
  }
  elsif ( $lvl == DBG ) {
    $severity = 'DBG ';
  }
  else {
    $severity = 'UNKN';
  }

  my $PID = $$;

  open LOG, ">>" . $logfile or die $!;
  my ($seconds, $microseconds) = gettimeofday();
  my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) = localtime($seconds);
  printf(LOG "%5d:%4d-%02d-%02d %02d:%02d:%02d.%03d - %s: %s\n", $PID, $year+1900,$mon+1,$mday,$hour,$min,$sec, $microseconds / 1000, $severity, $str);
  close(LOG);
}

sub getDatabaseConnection()
{
  my $conf_ref = shift;

  # Check if perl DBD/mysql is available
  eval {
    require DBD::mysql;
  };

  my $dbh = undef;
  if ( $@ ) {
    if ( $$conf_ref{'FetchMethod'} eq 'DBI' ) {
      &log($$conf_ref{'LogFile'}, WARN, "Warning: No DBD::mysql found. Using fallback method. Please configure fromdual mysql agent for zabbix accordingly.");
      $$conf_ref{'FetchMethod'} = 'mysql';
    }
  }
  else {
    $dbh = DBI->connect('DBI:mysql:database=mysql;host=' . $$conf_ref{'MysqlHost'} . ';port=' . $$conf_ref{'MysqlPort'}
                        , $$conf_ref{'Username'}, $$conf_ref{'Password'}
                        , { 'RaiseError' => 0, 'PrintError' => 0 }
                          );

    if ( defined $DBI::err ) {

      &log($$conf_ref{'LogFile'}, INFO, 'DBI connect with database=mysql, host=' . $$conf_ref{'MysqlHost'} . ', port=' . $$conf_ref{'MysqlPort'} . ' and user=' . $$conf_ref{'Username'} . ' failed: ' . $DBI::errstr);
      close(LOG);
    }
  }

  if ( $$conf_ref{'Debug'} ) {
    &log($$conf_ref{'LogFile'}, INFO, 'Call with ' . $$conf_ref{'FetchMethod'});
  }
  return $dbh;
}

sub releaseDatabaseConnection() {

  my $conf_ref = shift;
  my $dbh = shift;

  if ( $$conf_ref{'FetchMethod'} eq 'DBI' ) {
    $dbh->disconnect();
  }
}

sub initValues {

  my $variables_ref = shift;
  my $variables_to_send_ref = shift;

  foreach my $key ( @$variables_to_send_ref ) {
    $$variables_ref{$key} = 0;
  }
}

sub getGlobalVariables {

  my $conf_ref = shift;
  my $dbh = shift;
  my $variables_ref = shift;

  my $sql = 'SHOW GLOBAL VARIABLES';
  if ( $$conf_ref{'FetchMethod'} eq 'DBI' ) {

    my $sth = $dbh->prepare($sql);
    if ( $sth->execute() ) {
      while ( my $ref = $sth->fetchrow_hashref() ) {
        $$variables_ref{$ref->{'Variable_name'}} = $ref->{'Value'};
      }
      $sth->finish();
    }
  }
  elsif ( $$conf_ref{'FetchMethod'} eq 'mysql' ) {

    my $cmd = 'mysql --user=' . $$conf_ref{'Username'} . ' --host=' . $$conf_ref{'MysqlHost'} . ' --port=' . $$conf_ref{'MysqlPort'} . ' --password=' . $$conf_ref{'Password'} .' --execute="' . $sql . '"';
    if ( $$conf_ref{'Debug'} ) {
      &log($$conf_ref{'LogFile'}, DBG, "$cmd");
    }

    my @stdout = `$cmd 2>&1`;

    if ( $? != 0 ) {
      &log($$conf_ref{'LogFile'}, ERR, "$cmd failed.");
    }
    else {

      foreach my $line ( @stdout ) {

        if ( $line =~ m/^(\S+)\s+(.*)\s*$/ ) {
          if ( "$1" ne 'Variable_name' ) {
            $$variables_ref{$1} = "$2";
          }
        }
      }
    }
  }

  if ( $$conf_ref{'Debug'} == WARN ) {
    &log($$conf_ref{'LogFile'}, INFO, 'Global Variables:');
    foreach my $key ( keys %$variables_ref ) {
      &log($$conf_ref{'LogFile'}, DBG, "key: $key, value: $$variables_ref{$key}");
    }
  }
}

sub getGlobalStatus {

  my $conf_ref = shift;
  my $dbh = shift;
  my $status_ref = shift;

  my $sql = 'SHOW /*!50000 GLOBAL */ STATUS';
  if ( $$conf_ref{'FetchMethod'} eq 'DBI' ) {

    my $sth = $dbh->prepare($sql);
    if ( $sth->execute() ) {
      while ( my $ref = $sth->fetchrow_hashref() ) {
        $$status_ref{$ref->{'Variable_name'}} = $ref->{'Value'};
      }
      $sth->finish();
    }
  }
  elsif ( $$conf_ref{'FetchMethod'} eq 'mysql' ) {

    my $cmd = 'mysql --user=' . $$conf_ref{'Username'} . ' --host=' . $$conf_ref{'MysqlHost'} . ' --port=' . $$conf_ref{'MysqlPort'} . ' --password=' . $$conf_ref{'Password'} . " --execute='" . $sql . "'";
    if ( $$conf_ref{'Debug'} ) {
      &log($$conf_ref{'LogFile'}, DBG, "$cmd");
    }

    my @stdout = `$cmd 2>&1`;

    if ( $? != 0 ) {
      &log($$conf_ref{'LogFile'}, ERR, "$cmd failed.");
    }
    else {

      foreach my $line ( @stdout ) {

        if ( $line =~ m/^(\S+)\s+(.*)\s*$/ ) {
          if ( "$1" ne 'Variable_name' ) {
            $$status_ref{$1} = "$2";
          }
        }
      }
    }
  }

  if ( $$conf_ref{'Debug'} == WARN ) {
    &log($$conf_ref{'LogFile'}, INFO, 'Global Status:');
    foreach my $key ( keys %$status_ref ) {
      &log($$conf_ref{'LogFile'}, DBG, "key: $key, value: $$status_ref{$key}");
    }
  }
}

1;
__END__